@@ -104,7 +104,7 @@ TODO: Some way to do something about that?
 
 # TODO: Cache compiled ASTs!
 
-__version__ = "0.9.1"
+__version__ = "0.9.3"
 
 import sys
 import inspect
@@ -288,7 +288,7 @@ def captureOutput():
     output capturing is already in place.
     """
     global OUTPUT_CAPTURE, ORIGINAL_INPUT_FN
-    if OUTPUT_CAPTURE is None:
+    if OUTPUT_CAPTURE is None or sys.stdout != OUTPUT_CAPTURE:
         ORIGINAL_INPUT_FN = builtins.input
         builtins.input = _echoing_input
         OUTPUT_CAPTURE = CapturingStream()
@@ -579,61 +579,70 @@ def ellipsis(string, maxlen=40):
         return string
 
 
-def print_success_message(
+def print_message(msg, color=None):
+    """
+    Prints a test result message to sys.stderr, but also flushes stdout
+    and stderr both beforehand and afterwards to improve message
+    ordering.
+
+    If a color is given, it should be an ANSI terminal color code string
+    (just the digits, for example '34' for blue or '1;31' for bright red).
+    """
+    sys.stdout.flush()
+    sys.stderr.flush()
+
+    # Make the whole message blue
+    if color:
+        print(f"\x1b[{color}m", end="", file=sys.stderr)
+        suffix = "\x1b[0m"
+    else:
+        suffix = ""
+
+    print(msg + suffix, file=sys.stderr)
+
+    sys.stdout.flush()
+    sys.stderr.flush()
+
+
+def create_success_message(
     tag,
     context,
     details,
     include_expr_details=True
 ):
     """
-    Prints an expectation success message for the given test context,
-    with the given details to be printed if the detail level is 1 or
-    higher. Unless `include_expr_details` is set to False, details of
-    the test expression will also be printed (but only when the detail
+    Returns an expectation success message (a string) for the given test
+    context, with the given details to be printed if the detail level is
+    1 or higher. Unless `include_expr_details` is set to False, details
+    of the test expression will also be included (but only when the detail
     level is at least 1). The tag should be a filename:lineno string
     indicating where the expectation originated.
     """
-    # Flush stdout & stderr to improve ordering
-    sys.stdout.flush()
-    sys.stderr.flush()
-
     # Tags for us and our test
     SUMMARY_INFO["met"].append(tag)
     ctag = f"{context['file']}:{context['line']}"
 
-    # Make the whole message blue
-    if COLORS:
-        print("\x1b[34m", end="", file=sys.stderr)
-        suffix = "\x1b[0m"
-    else:
-        suffix = ""
-
     # Detail level 1 gives more output for successes
     if DETAIL_LEVEL < 1:
-        print(f"✓ {tag}{suffix}", file=sys.stderr)
+        result = f"✓ {tag}"
     else:
-        print(
-            f"✓ expectation from {tag} met for test at {ctag}",
-            file=sys.stderr
-        )
+        result = f"✓ expectation from {tag} met for test at {ctag}"
         detail_msg = indent(details, 2)
-        if detail_msg.startswith('\n'):
-            detail_msg = detail_msg[1:]
+        if not detail_msg.startswith('\n'):
+            detail_msg = '\n' + detail_msg
 
         # Expression details unless suppressed
         if include_expr_details:
             expr_base, expr_extra = expr_details(context)
-            detail_msg += indent(expr_base, 2)
-            detail_msg += indent(expr_extra, 2)
+            detail_msg += '\n' + indent(expr_base, 2)
+            detail_msg += '\n' + indent(expr_extra, 2)
 
-        print(f"{detail_msg}{suffix}", file=sys.stderr)
+        result += detail_msg
 
-    # Flush stdout & stderr to improve ordering
-    sys.stdout.flush()
-    sys.stderr.flush()
+    return result
 
 
-def print_failure_message(
+def create_failure_message(
     tag,
     context,
     details,
@@ -641,66 +650,42 @@ def print_failure_message(
     include_expr_details=True
 ):
     """
-    Prints an expectation failure message for the given test context,
-    with the given details to be printed if the detail level is 0 or
-    higher, and the given extra details to also be printed if the detail
-    level is 1 or higher. Unless `include_expr_details` is set to false,
-    details of the expression tested will be included automatically. The
-    tag should be a filename:lineno string indicating where the
-    expectation originated.
+    Returns an expectation failure message (a string) for the given test
+    context, with the given details to be printed if the detail level is
+    0 or higher, and the given extra details to also be printed if the
+    detail level is 1 or higher. Unless `include_expr_details` is set to
+    false, details of the expression tested will be included
+    automatically. The tag should be a filename:lineno string indicating
+    where the expectation originated.
     """
-    # Flush stdout & stderr to improve ordering
-    sys.stdout.flush()
-    sys.stderr.flush()
     # Tags for us and our test
     SUMMARY_INFO["unmet"].append(tag)
     ctag = f"{context['file']}:{context['line']}"
 
-    # Make the whole message bold and red
-    if COLORS:
-        print("\x1b[1;31m", end="", file=sys.stderr)
-        suffix = "\x1b[0m"
-    else:
-        suffix = ""
-
     # Detail level controls initial message
     if DETAIL_LEVEL < 1:
-        print(f"✗ {tag}", file=sys.stderr)
+        result = f"✗ {tag}"
     else:
-        print(
-            f"✗ expectation from {tag} NOT met for test at {ctag}",
-            file=sys.stderr
-        )
+        result = f"✗ expectation from {tag} NOT met for test at {ctag}"
 
     # Assemble our details message
     detail_msg = ''
 
     # Detail level controls printing of detail messages
     if DETAIL_LEVEL >= 0:
-        if not detail_msg.endswith('\n'):
-            detail_msg += '\n'
-        detail_msg += indent(details, 2)
+        detail_msg += '\n' + indent(details, 2)
     if DETAIL_LEVEL >= 1 and extra_details:
-        if not detail_msg.endswith('\n'):
-            detail_msg += '\n'
-        detail_msg += indent(extra_details, 2)
+        detail_msg += '\n' + indent(extra_details, 2)
 
     # Expression details unless suppressed
     if include_expr_details:
         expr_base, expr_extra = expr_details(context)
         if DETAIL_LEVEL >= 0:
-            detail_msg += indent(expr_base, 2)
+            detail_msg += '\n' + indent(expr_base, 2)
         if DETAIL_LEVEL >= 1:
-            detail_msg += indent(expr_extra, 2)
+            detail_msg += '\n' + indent(expr_extra, 2)
 
-    if detail_msg.startswith('\n'):
-        detail_msg = detail_msg[1:]
-
-    print(f"{detail_msg}{suffix}", file=sys.stderr)
-
-    # Flush stdout & stderr to improve ordering
-    sys.stdout.flush()
-    sys.stderr.flush()
+    return result + detail_msg
 
 
 def expr_details(context):
@@ -717,7 +702,7 @@ def expr_details(context):
 
     # Base message
     if context.get("relevant"):
-        msg += f"\nIn expression {short_expr}, values were:"
+        msg += f"In expression {short_expr}, values were:"
         for key in sorted(
             context["relevant"],
             key=lambda fragment: expr.index(fragment)
@@ -725,7 +710,7 @@ def expr_details(context):
             val = context.get("values", {}).get(key, "???")
             msg += f"\n  {key} = {val}"
     else:
-        msg += f"\nIn expression {short_expr}, values were:"
+        msg += f"In expression {short_expr}, values were:"
         vdict = context.get("values", {})
         for key in sorted(
             vdict.keys(),
@@ -736,7 +721,9 @@ def expr_details(context):
 
     # Extra message
     if short_expr != expr:
-        extra_msg += f"\nFull expression:\n  {expr}"
+        if not extra_msg.endswith('\n'):
+            extra_msg += '\n'
+        extra_msg += f"Full expression:\n  {expr}"
     extra_values = sorted(
         [
             key
@@ -746,7 +733,9 @@ def expr_details(context):
         key=lambda fragment: expr.index(fragment)
     )
     if context.get("relevant") and extra_values:
-        extra_msg += "\nExtra values:"
+        if not extra_msg.endswith('\n'):
+            extra_msg += '\n'
+        extra_msg += "Extra values:"
         for ev in extra_values:
             val = context.get("values", {}).get(ev, "???")
             extra_msg += f"\n  {ev} = {val}"
@@ -767,8 +756,9 @@ def check_expectation(exp, set_success=True):
 
     This function will add a "met" key to the expectation dictionary it
     checks, unless the `set_success` argument is set to False. It will
-    also print a success or failure message, and will return True on
-    success and False on failure.
+    also print a success or failure message, and add that message as the
+    "message" key (again unless `set_success` is False). It will return
+    True on success and False on failure.
     """
     case = exp["case"]
     context = case["context"]
@@ -780,7 +770,7 @@ def check_expectation(exp, set_success=True):
         full_result = repr(result)
         full_expected = repr(expected)
         if result == expected:
-            print_success_message(
+            msg = create_success_message(
                 exp["tag"],
                 context,
                 (
@@ -788,8 +778,10 @@ def check_expectation(exp, set_success=True):
                     f" expected value {short_expected}."
                 )
             )
+            print_message(msg, color="34" if COLORS else None)
             if set_success:
                 exp["met"] = True
+                exp["message"] = msg
             return True
         else:
             # Construct base report
@@ -804,9 +796,16 @@ def check_expectation(exp, set_success=True):
                 f"Full expectation:\n  {full_expected}"
             )
 
-            print_failure_message(exp["tag"], context, msg, extra_msg)
+            msg = create_failure_message(
+                exp["tag"],
+                context,
+                msg,
+                extra_msg
+            )
+            print_message(msg, color='1;31' if COLORS else None)
             if set_success:
                 exp["met"] = False
+                exp["message"] = msg
             return False
     elif exp["type"] == "output":
         case = exp["case"]
@@ -814,7 +813,7 @@ def check_expectation(exp, set_success=True):
         output = case["output"]
         fragment = exp["value"]
         if output is None:
-            print_failure_message(
+            msg = create_failure_message(
                 exp["tag"],
                 context,
                 "No output has been captured.",
@@ -824,14 +823,17 @@ def check_expectation(exp, set_success=True):
                 ),
                 False
             )
-            exp["met"] = False
+            print_message(msg, color='1;31' if COLORS else None)
+            if set_success:
+                exp["met"] = False
+                exp["message"] = msg
             return False
         short_output = ellipsis(output)
         short_fragment = ellipsis(fragment)
         full_output = output
         full_fragment = fragment
         if fragment in output:
-            print_success_message(
+            msg = create_success_message(
                 exp["tag"],
                 context,
                 (
@@ -839,8 +841,10 @@ def check_expectation(exp, set_success=True):
                     f' recorded output """{short_output}""".'
                 )
             )
+            print_message(msg, color='34' if COLORS else None)
             if set_success:
                 exp["met"] = True
+                exp["message"] = msg
             return True
         else:
             # Construct base report
@@ -861,9 +865,11 @@ def check_expectation(exp, set_success=True):
             if extra_msg.startswith('\n'):
                 extra_msg = extra_msg[1:]
 
-            print_failure_message(exp["tag"], context, msg, extra_msg)
+            msg = create_failure_message(exp["tag"], context, msg, extra_msg)
+            print_message(msg, color='1;31' if COLORS else None)
             if set_success:
                 exp["met"] = False
+                exp["message"] = msg
             return False
     else: # we assume it's a custom expectation
         case = exp["case"]
@@ -883,7 +889,7 @@ def check_expectation(exp, set_success=True):
         test_name = tester.__name__
 
         if test_result is True:
-            print_success_message(
+            msg = create_success_message(
                 exp["tag"],
                 context,
                 (
@@ -891,8 +897,10 @@ def check_expectation(exp, set_success=True):
                     f' {short_result}{output_clause}.'
                 )
             )
+            print_message(msg, color='34' if COLORS else None)
             if set_success:
                 exp["met"] = True
+                exp["message"] = msg
             return True
         else:
             # Construct base report
@@ -908,9 +916,11 @@ def check_expectation(exp, set_success=True):
             if output != ellipsis(output):
                 extra_msg += f"\nFull output:\n  {output}"
 
-            print_failure_message(exp["tag"], context, msg, extra_msg)
+            msg = create_failure_message(exp["tag"], context, msg, extra_msg)
+            print_message(msg, color='1;31' if COLORS else None)
             if set_success:
                 exp["met"] = False
+                exp["message"] = msg
             return False
 
 
@@ -925,12 +935,13 @@ def expectResult(expected):
     """
     if CURRENT_CASE is None:
         context = get_my_location()
-        print_failure_message(
+        msg = create_failure_message(
             context,
             "No test case has been established!",
             "You must call testCase before calling expectResult.",
             False
         )
+        print_message(msg, color='1;31' if COLORS else None)
         return False
     else:
         myloc = get_my_location()
@@ -955,12 +966,13 @@ def expectOutputContains(fragment):
     """
     if CURRENT_CASE is None:
         context = get_my_location()
-        print_failure_message(
+        msg = create_failure_message(
             context,
             "No test case has been established!",
             "You must call testCase before calling expectOutputContains.",
             False
         )
+        print_message(msg, color='1;31' if COLORS else None)
         return False
     else:
         myloc = get_my_location()
@@ -991,12 +1003,13 @@ def expectCustom(tester):
     """
     if CURRENT_CASE is None:
         context = get_my_location()
-        print_failure_message(
+        msg = create_failure_message(
             context,
             "No test case has been established!",
             "You must call testCase before calling expectCustom.",
             False
         )
+        print_message(msg, color='1;31' if COLORS else None)
         return False
     else:
         myloc = get_my_location()
@@ -1667,8 +1680,8 @@ def recheck_expectation(expectation, new_context):
     previous context.
 
     This function will print out a success or failure message similar to
-    the original message, and will return True or False depending on
-    whether the expectation is met.
+    the original message, and will return a new expectation dictionary
+    including "met" and "message" keys based on the new evaluation.
     """
     sys.stderr.flush()
     case = expectation["case"]
@@ -1720,4 +1733,7 @@ def recheck_expectation(expectation, new_context):
     new_expectation.update(expectation)
     new_expectation["case"] = new_case
 
-    return check_expectation(new_expectation, set_success=False)
+    # Result will be placed in the "met" key of the new expectation
+    _ = check_expectation(new_expectation, set_success=True)
+
+    return new_expectation
